==> oxapy-0.4.5.tar.gz: oxapy-0.4.5/src/handling/request_handler.rs <==
use std::{collections::HashMap, mem::transmute, sync::Arc};

use http_body_util::{BodyExt, Full};
use hyper::{
    body::{Bytes, Incoming},
    Request as HyperRequest, Response as HyperResponse,
};
use pyo3::{Py, PyAny};
use tokio::sync::mpsc::{channel, Sender};

use crate::{
    cors::Cors,
    multipart::{parse_mutltipart, MultiPart},
    request::Request,
    response::Response,
    routing::Router,
    session::SessionStore,
    status::Status,
    templating::Template,
    IntoPyException, MatchitRoute, ProcessRequest,
};

fn convert_to_hyper_response(
    response: Response,
) -> Result<HyperResponse<Full<Bytes>>, hyper::http::Error> {
    let mut response_builder = HyperResponse::builder().status(response.status as u16);
    for (key, value) in response.headers {
        response_builder = response_builder.header(key, value);
    }
    response_builder.body(Full::new(response.body))
}

#[allow(clippy::too_many_arguments)]
pub async fn handle_request(
    req: HyperRequest<Incoming>,
    request_sender: Sender<ProcessRequest>,
    routers: Vec<Arc<Router>>,
    app_data: Option<Arc<Py<PyAny>>>,
    channel_capacity: usize,
    cors: Option<Arc<Cors>>,
    template: Option<Arc<Template>>,
    session_store: Option<Arc<SessionStore>>,
) -> Result<HyperResponse<Full<Bytes>>, hyper::http::Error> {
    if req.method() == hyper::Method::OPTIONS && cors.is_some() {
        let response = cors.unwrap().as_ref().clone().into();
        return convert_to_hyper_response(response);
    }

    let request = convert_hyper_request(req, app_data, template, session_store)
        .await
        .unwrap();

    for router in &routers {
        if let Some(route) = router.find(&request.method, &request.uri) {
            let (response_sender, mut respond_receive) = channel(channel_capacity);

            let route: MatchitRoute = unsafe { transmute(&route) };

            let process_request = ProcessRequest {
                request: request.clone(),
                router: router.clone(),
                route,
                response_sender,
                cors: cors.clone(),
            };

            if request_sender.send(process_request).await.is_ok() {
                if let Some(response) = respond_receive.recv().await {
                    return convert_to_hyper_response(response);
                }
            }
            break;
        }
    }

    let response = if let Some(cors_config) = cors {
        cors_config
            .apply_to_response(Status::NOT_FOUND.into())
            .unwrap()
    } else {
        Status::NOT_FOUND.into()
    };

    convert_to_hyper_response(response)
}

fn extract_session_id_from_cookie(
    cookie_header: Option<&String>,
    cookie_name: &str,
) -> Option<String> {
    cookie_header.and_then(|cookies| {
        cookies
            .split(';')
            .filter_map(|cookie| {
                let cookie = cookie.trim();
                let mut parts = cookie.splitn(2, '=');
                if let (Some(name), Some(value)) = (
                    parts.next().map(|s| s.trim()),
                    parts.next().map(|s| s.trim()),
                ) {
                    if name == cookie_name {
                        return Some(value.to_string());
                    }
                }
                None
            })
            .next()
    })
}

async fn convert_hyper_request(
    req: HyperRequest<Incoming>,
    app_data: Option<Arc<Py<PyAny>>>,
    template: Option<Arc<Template>>,
    session_store: Option<Arc<SessionStore>>,
) -> Result<Arc<Request>, Box<dyn std::error::Error + Sync + Send>> {
    let method = req.method().to_string();
    let uri = req.uri().to_string();

    let mut headers = HashMap::new();
    for (key, value) in req.headers() {
        headers.insert(
            key.to_string(),
            value.to_str().unwrap_or_default().to_string(),
        );
    }

    let mut request = Request::new(method, uri, headers.clone());

    if let Some(ref store) = session_store {
        let session_id = extract_session_id_from_cookie(headers.get("cookie"), &store.cookie_name);

        let session = store.get_session(session_id).map_err(|e| {
            Box::new(std::io::Error::new(
                std::io::ErrorKind::Other,
                format!("Failed to get session: {}", e),
            ))
        })?;
        request.session = Some(Arc::new(session));
        request.session_store = Some(store.clone());
    }

    let body_bytes = req.collect().await?.to_bytes();
    let body = String::from_utf8_lossy(&body_bytes).to_string();

    if let Some(content_type) = headers.get("content-type") {
        if content_type.starts_with("multipart/form-data") {
            let MultiPart { fields, files } = parse_mutltipart(content_type, body_bytes)
                .await
                .into_py_exception()?;
            request.form_data = Some(fields);
            request.files = Some(files);
        }
    }

    if !body.is_empty() {
        request.body = Some(body);
    }

    request.app_data = app_data;
    request.template = template;

    Ok(Arc::new(request))
}

==> oxapy-0.4.6.tar.gz: oxapy-0.4.6/src/handling/request_handler.rs <==
use std::{collections::HashMap, mem::transmute, sync::Arc};

use http_body_util::{BodyExt, Full};
use hyper::{
    body::{Bytes, Incoming},
    Request as HyperRequest, Response as HyperResponse,
};
use pyo3::{Py, PyAny};
use tokio::sync::mpsc::{channel, Sender};

use crate::{
    cors::Cors,
    multipart::{parse_mutltipart, MultiPart},
    request::Request,
    response::Response,
    routing::Router,
    session::SessionStore,
    status::Status,
    templating::Template,
    IntoPyException, MatchitRoute, ProcessRequest,
};

fn convert_to_hyper_response(
    response: Response,
) -> Result<HyperResponse<Full<Bytes>>, hyper::http::Error> {
    let mut response_builder = HyperResponse::builder().status(response.status as u16);
    for (key, value) in response.headers {
        response_builder = response_builder.header(key, value);
    }
    response_builder.body(Full::new(response.body))
}

#[allow(clippy::too_many_arguments)]
pub async fn handle_request(
    req: HyperRequest<Incoming>,
    request_sender: Sender<ProcessRequest>,
    routers: Vec<Arc<Router>>,
    app_data: Option<Arc<Py<PyAny>>>,
    channel_capacity: usize,
    cors: Option<Arc<Cors>>,
    template: Option<Arc<Template>>,
    session_store: Option<Arc<SessionStore>>,
) -> Result<HyperResponse<Full<Bytes>>, hyper::http::Error> {
    if req.method() == hyper::Method::OPTIONS && cors.is_some() {
        let response = cors.unwrap().as_ref().clone();
        return convert_to_hyper_response(response.into());
    }

    let request = convert_hyper_request(req, app_data, template, session_store)
        .await
        .unwrap();

    for router in &routers {
        if let Some(route) = router.find(&request.method, &request.uri) {
            let (response_sender, mut respond_receive) = channel(channel_capacity);

            let route: MatchitRoute = unsafe { transmute(&route) };

            let process_request = ProcessRequest {
                request: request.clone(),
                router: router.clone(),
                route,
                response_sender,
                cors: cors.clone(),
            };

            if request_sender.send(process_request).await.is_ok() {
                if let Some(response) = respond_receive.recv().await {
                    return convert_to_hyper_response(response);
                }
            }
            break;
        }
    }

    let response = if let Some(cors_config) = cors {
        cors_config
            .apply_to_response(Status::NOT_FOUND.into())
            .unwrap()
    } else {
        Status::NOT_FOUND.into()
    };

    convert_to_hyper_response(response)
}

fn extract_session_id_from_cookie(
    cookie_header: Option<&String>,
    cookie_name: &str,
) -> Option<String> {
    cookie_header.and_then(|cookies| {
        cookies
            .split(';')
            .filter_map(|cookie| {
                let cookie = cookie.trim();
                let mut parts = cookie.splitn(2, '=');
                if let (Some(name), Some(value)) = (
                    parts.next().map(|s| s.trim()),
                    parts.next().map(|s| s.trim()),
                ) {
                    if name == cookie_name {
                        return Some(value.to_string());
                    }
                }
                None
            })
            .next()
    })
}

async fn convert_hyper_request(
    req: HyperRequest<Incoming>,
    app_data: Option<Arc<Py<PyAny>>>,
    template: Option<Arc<Template>>,
    session_store: Option<Arc<SessionStore>>,
) -> Result<Arc<Request>, Box<dyn std::error::Error + Sync + Send>> {
    let method = req.method().to_string();
    let uri = req.uri().to_string();

    let mut headers = HashMap::new();
    for (key, value) in req.headers() {
        headers.insert(
            key.to_string(),
            value.to_str().unwrap_or_default().to_string(),
        );
    }

    let mut request = Request::new(method, uri, headers.clone());

    if let Some(ref store) = session_store {
        let session_id = extract_session_id_from_cookie(headers.get("cookie"), &store.cookie_name);

        let session = store.get_session(session_id).map_err(|e| {
            Box::new(std::io::Error::new(
                std::io::ErrorKind::Other,
                format!("Failed to get session: {}", e),
            ))
        })?;
        request.session = Some(Arc::new(session));
        request.session_store = Some(store.clone());
    }

    let body_bytes = req.collect().await?.to_bytes();
    let body = String::from_utf8_lossy(&body_bytes).to_string();

    if let Some(content_type) = headers.get("content-type") {
        if content_type.starts_with("multipart/form-data") {
            let MultiPart { fields, files } = parse_mutltipart(content_type, body_bytes)
                .await
                .into_py_exception()?;
            request.form_data = Some(fields);
            request.files = Some(files);
        }
    }

    if !body.is_empty() {
        request.body = Some(body);
    }

    request.app_data = app_data;
    request.template = template;

    Ok(Arc::new(request))
}
